'''Package containing support code for pybald, static servers / etc'''


from .text import (camel_to_underscore,
                  underscore_to_camel,
                  ordinal_suffix,
                  pluralize,
                  strip_accents)
