
sample_config = True
env_name = "SampleTestProjectEnvironment"
template_path = "tests/sample_project/"
cache_path = None
project_name = "Sample Project"
debug = True
path = ""
template_helpers = ['from pybald.core.helpers import as_p, anchor, plural, js_escape']
database_engine_uri = 'sqlite:///:memory:'

